@@ -12,7 +12,7 @@
 ##############################################################################
 """Access control support
 
-$Id: Role.py 77297 2007-07-02 10:14:18Z shh $
+$Id: Role.py 88052 2008-07-06 01:14:06Z tseaver $
 """
 from cgi import escape
 
@@ -541,7 +541,8 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
 
     def possible_permissions(self):
         d={}
-        for p in Products.__ac_permissions__:
+        Products_permissions = getattr(Products, '__ac_permissions__', ())
+        for p in Products_permissions:
             d[p[0]]=1
         for p in self.aq_acquire('_getProductRegistryData')('ac_permissions'):
             d[p[0]]=1
