@@ -4,7 +4,7 @@
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
@@ -328,7 +328,7 @@
    http://www.bell-labs.com/people/cope/oopsla/Oopsla96TechnicalProgramAbstracts.html#GilLorenz, 
    OOPSLA '96 Proceedings, ACM SIG-PLAN, October, 1996
 
-$Id: tests.py 30282 2005-05-06 15:08:46Z jim $
+$Id: tests.py 99193 2009-04-15 14:44:25Z witsch $
 """
 
 import ExtensionClass
@@ -1502,6 +1502,32 @@ def test_cant_pickle_acquisition_wrappers_newstyle():
     TypeError: Can't pickle objects in acquisition wrappers.
     """
 
+def test_z3interfaces():
+    """
+    >>> from zope.interface.verify import verifyClass
+
+    Explicit and Implicit implement IAcquirer:
+
+    >>> from Acquisition import Explicit
+    >>> from Acquisition import Implicit
+    >>> from Acquisition.interfaces import IAcquirer
+    >>> verifyClass(IAcquirer, Explicit)
+    True
+    >>> verifyClass(IAcquirer, Implicit)
+    True
+
+    ExplicitAcquisitionWrapper and ImplicitAcquisitionWrapper implement
+    IAcquisitionWrapper:
+
+    >>> from Acquisition import ExplicitAcquisitionWrapper
+    >>> from Acquisition import ImplicitAcquisitionWrapper
+    >>> from Acquisition.interfaces import IAcquisitionWrapper
+    >>> verifyClass(IAcquisitionWrapper, ExplicitAcquisitionWrapper)
+    True
+    >>> verifyClass(IAcquisitionWrapper, ImplicitAcquisitionWrapper)
+    True
+    """
+
 def show(x):
     print showaq(x).strip()
     
@@ -1594,7 +1620,127 @@ def test_Wrapper_gc():
 """
 
 
-    
+def test_proxying():
+    """Make sure that recent python slots are proxied.
+
+    >>> import Acquisition
+    >>> class Impl(Acquisition.Implicit):
+    ...     pass
+
+    >>> class C(Acquisition.Implicit):
+    ...     def __getitem__(self, key):
+    ...         print 'getitem', key
+    ...         if key == 4:
+    ...             raise IndexError
+    ...         return key
+    ...     def __contains__(self, key):
+    ...         print 'contains', repr(key)
+    ...         return key == 5
+    ...     def __iter__(self):
+    ...         print 'iterating...'
+    ...         return iter((42,))
+
+    The naked class behaves like this:
+
+    >>> c = C()
+    >>> 3 in c
+    contains 3
+    False
+    >>> 5 in c
+    contains 5
+    True
+    >>> list(c)
+    iterating...
+    [42]
+
+    Let's put c in the context of i:
+
+    >>> i = Impl()
+    >>> i.c = c
+
+    Now check that __contains__ is properly used:
+
+    >>> 3 in i.c # c.__of__(i)
+    contains 3
+    False
+    >>> 5 in i.c
+    contains 5
+    True
+    >>> list(i.c)
+    iterating...
+    [42]
+
+    Let's let's test the same again with an explicit wrapper:
+
+    >>> import Acquisition
+    >>> class Impl(Acquisition.Explicit):
+    ...     pass
+
+    >>> class C(Acquisition.Explicit):
+    ...     def __getitem__(self, key):
+    ...         print 'getitem', key
+    ...         if key == 4:
+    ...             raise IndexError
+    ...         return key
+    ...     def __contains__(self, key):
+    ...         print 'contains', repr(key)
+    ...         return key == 5
+    ...     def __iter__(self):
+    ...         print 'iterating...'
+    ...         return iter((42,))
+
+    The naked class behaves like this:
+
+    >>> c = C()
+    >>> 3 in c
+    contains 3
+    False
+    >>> 5 in c
+    contains 5
+    True
+    >>> list(c)
+    iterating...
+    [42]
+
+    Let's put c in the context of i:
+
+    >>> i = Impl()
+    >>> i.c = c
+
+    Now check that __contains__ is properly used:
+
+    >>> 3 in i.c # c.__of__(i)
+    contains 3
+    False
+    >>> 5 in i.c
+    contains 5
+    True
+    >>> list(i.c)
+    iterating...
+    [42]
+
+    Finally let's check that https://bugs.launchpad.net/zope2/+bug/360761
+    has been fixed:
+
+    >>> class C(Acquisition.Implicit):
+    ...     l=[1,2,3]
+    ...     def __getitem__(self, i):
+    ...         return self.l[i]
+
+    >>> c1 = C()
+    >>> type(iter(c1))
+    <type 'iterator'>
+    >>> list(c1)
+    [1, 2, 3]
+
+    >>> c2 = C().__of__(c1)
+    >>> type(iter(c2))
+    <type 'iterator'>
+    >>> list(c2)
+    [1, 2, 3]
+
+    """
+
 
 import unittest
 from zope.testing.doctest import DocTestSuite
@@ -1604,4 +1750,5 @@ def test_suite():
         DocTestSuite(),
         ))
 
-if __name__ == '__main__': unittest.main()
+if __name__ == '__main__':
+    unittest.main(defaultTest='test_suite')
