@@ -1,9 +1,9 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
@@ -12,13 +12,20 @@
 ##############################################################################
 """Image object
 
-$Id: Image.py 65863 2006-03-07 21:54:19Z shh $
+$Id: Image.py 69771 2006-08-24 16:52:47Z shh $
 """
-import Globals, struct
-from OFS.content_types import guess_content_type
+import struct
+from zope.contenttype import guess_content_type
 from Globals import DTMLFile
+from Globals import InitializeClass
 from PropertyManager import PropertyManager
+from AccessControl import ClassSecurityInfo
 from AccessControl.Role import RoleManager
+from AccessControl.Permissions import change_images_and_files
+from AccessControl.Permissions import view_management_screens
+from AccessControl.Permissions import view as View
+from AccessControl.Permissions import ftp_access
+from AccessControl.Permissions import delete_objects
 from webdav.common import rfc1123_date
 from webdav.Lockable import ResourceLockedError
 from webdav.WriteLockInterface import WriteLockInterface
@@ -36,7 +43,6 @@ from zExceptions import Redirect
 from cgi import escape
 import transaction
 
-StringType=type('')
 manage_addFileForm=DTMLFile('dtml/imageAdd', globals(),Kind='File',kind='file')
 def manage_addFile(self,id,file='',title='',precondition='', content_type='',
                    REQUEST=None):
@@ -74,14 +80,18 @@ class File(Persistent, Implicit, PropertyManager,
     __implements__ = (WriteLockInterface, HTTPRangeSupport.HTTPRangeInterface)
     meta_type='File'
 
+    security = ClassSecurityInfo()
+    security.declareObjectProtected(View)
 
     precondition=''
     size=None
-    alt=''
 
     manage_editForm  =DTMLFile('dtml/fileEdit',globals(),
                                Kind='File',kind='file')
     manage_editForm._setName('manage_editForm')
+
+    security.declareProtected(view_management_screens, 'manage')
+    security.declareProtected(view_management_screens, 'manage_main')
     manage=manage_main=manage_editForm
     manage_uploadForm=manage_editForm
 
@@ -98,24 +108,7 @@ class File(Persistent, Implicit, PropertyManager,
         + Cacheable.manage_options
         )
 
-
-    __ac_permissions__=(
-        ('View management screens',
-         ('manage', 'manage_main',)),
-        ('Change Images and Files',
-         ('manage_edit','manage_upload','PUT')),
-        ('View',
-         ('index_html', 'view_image_or_file', 'get_size',
-          'getContentType', 'PrincipiaSearchSource', '')),
-        ('FTP access',
-         ('manage_FTPstat','manage_FTPget','manage_FTPlist')),
-        ('Delete objects',
-         ('DELETE',)),
-        )
-
-
     _properties=({'id':'title', 'type': 'string'},
-                 {'id':'alt', 'type':'string'},
                  {'id':'content_type', 'type':'string'},
                  )
 
@@ -235,7 +228,7 @@ class File(Persistent, Implicit, PropertyManager,
                     RESPONSE.setStatus(206) # Partial content
 
                     data = self.data
-                    if type(data) is StringType:
+                    if isinstance(data, str):
                         RESPONSE.write(data[start:end])
                         return True
 
@@ -306,7 +299,7 @@ class File(Persistent, Implicit, PropertyManager,
                             'Content-Range: bytes %d-%d/%d\r\n\r\n' % (
                                 start, end - 1, self.size))
 
-                        if type(data) is StringType:
+                        if isinstance(data, str):
                             RESPONSE.write(data[start:end])
 
                         else:
@@ -355,6 +348,7 @@ class File(Persistent, Implicit, PropertyManager,
                     RESPONSE.write('\r\n--%s--\r\n' % boundary)
                     return True
 
+    security.declareProtected(View, 'index_html')
     def index_html(self, REQUEST, RESPONSE):
         """
         The default view of the contents of a File or Image.
@@ -404,7 +398,7 @@ class File(Persistent, Implicit, PropertyManager,
         self.ZCacheable_set(None)
 
         data=self.data
-        if type(data) is type(''):
+        if isinstance(data, str):
             RESPONSE.setBase(None)
             return data
 
@@ -414,12 +408,14 @@ class File(Persistent, Implicit, PropertyManager,
 
         return ''
 
+    security.declareProtected(View, 'view_image_or_file')
     def view_image_or_file(self, URL1):
         """
         The default view of the contents of the File or Image.
         """
         raise Redirect, URL1
 
+    security.declareProtected(View, 'PrincipiaSearchSource')
     def PrincipiaSearchSource(self):
         """ Allow file objects to be searched.
         """
@@ -427,9 +423,12 @@ class File(Persistent, Implicit, PropertyManager,
             return str(self.data)
         return ''
 
-    # private
-    update_data__roles__=()
+    security.declarePrivate('update_data')
     def update_data(self, data, content_type=None, size=None):
+        if isinstance(data, unicode):
+            raise TypeError('Data can only be str or file-like.  '
+                            'Unicode objects are expressly forbidden.')
+
         if content_type is not None: self.content_type=content_type
         if size is None: size=len(data)
         self.size=size
@@ -438,6 +437,7 @@ class File(Persistent, Implicit, PropertyManager,
         self.ZCacheable_set(None)
         self.http__refreshEtag()
 
+    security.declareProtected(change_images_and_files, 'manage_edit')
     def manage_edit(self, title, content_type, precondition='',
                     filedata=None, REQUEST=None):
         """
@@ -458,6 +458,7 @@ class File(Persistent, Implicit, PropertyManager,
             message="Saved changes."
             return self.manage_main(self,REQUEST,manage_tabs_message=message)
 
+    security.declareProtected(change_images_and_files, 'manage_upload')
     def manage_upload(self,file='',REQUEST=None):
         """
         Replaces the current contents of the File or Image object with file.
@@ -481,7 +482,7 @@ class File(Persistent, Implicit, PropertyManager,
         if headers and headers.has_key('content-type'):
             content_type=headers['content-type']
         else:
-            if type(body) is not type(''): body=body.data
+            if not isinstance(body, str): body=body.data
             content_type, enc=guess_content_type(
                 getattr(file, 'filename',id), body, content_type)
         return content_type
@@ -490,7 +491,7 @@ class File(Persistent, Implicit, PropertyManager,
 
         n=1 << 16
 
-        if type(file) is StringType:
+        if isinstance(file, str):
             size=len(file)
             if size < n: return file, size
             # Big string: cut it into smaller chunks
@@ -553,6 +554,9 @@ class File(Persistent, Implicit, PropertyManager,
 
         return next, size
 
+    security.declareProtected(delete_objects, 'DELETE')
+
+    security.declareProtected(change_images_and_files, 'PUT')
     def PUT(self, REQUEST, RESPONSE):
         """Handle HTTP PUT requests"""
         self.dav__init(REQUEST, RESPONSE)
@@ -569,6 +573,7 @@ class File(Persistent, Implicit, PropertyManager,
         RESPONSE.setStatus(204)
         return RESPONSE
 
+    security.declareProtected(View, 'get_size')
     def get_size(self):
         """Get the size of a file or image.
 
@@ -581,6 +586,7 @@ class File(Persistent, Implicit, PropertyManager,
     # deprecated; use get_size!
     getSize=get_size
 
+    security.declareProtected(View, 'getContentType')
     def getContentType(self):
         """Get the content type of a file or image.
 
@@ -592,6 +598,10 @@ class File(Persistent, Implicit, PropertyManager,
     def __str__(self): return str(self.data)
     def __len__(self): return 1
 
+    security.declareProtected(ftp_access, 'manage_FTPstat')
+    security.declareProtected(ftp_access, 'manage_FTPlist')
+
+    security.declareProtected(ftp_access, 'manage_FTPget')
     def manage_FTPget(self):
         """Return body for ftp."""
         RESPONSE = self.REQUEST.RESPONSE
@@ -608,7 +618,7 @@ class File(Persistent, Implicit, PropertyManager,
                 return result
 
         data = self.data
-        if type(data) is type(''):
+        if isinstance(data, str):
             RESPONSE.setBase(None)
             return data
 
@@ -719,25 +729,27 @@ class Image(File):
     __implements__ = (WriteLockInterface,)
     meta_type='Image'
 
+    security = ClassSecurityInfo()
+    security.declareObjectProtected(View)
 
+    alt=''
     height=''
     width=''
 
-    __ac_permissions__=(
-        ('View management screens',
-         ('manage', 'manage_main',)),
-        ('Change Images and Files',
-         ('manage_edit','manage_upload','PUT')),
-        ('View',
-         ('index_html', 'tag', 'view_image_or_file', 'get_size',
-          'getContentType', '')),
-        ('FTP access',
-         ('manage_FTPstat','manage_FTPget','manage_FTPlist')),
-        ('Delete objects',
-         ('DELETE',)),
-        )
+    # FIXME: Redundant, already in base class
+    security.declareProtected(change_images_and_files, 'manage_edit')
+    security.declareProtected(change_images_and_files, 'manage_upload')
+    security.declareProtected(change_images_and_files, 'PUT')
+    security.declareProtected(View, 'index_html')
+    security.declareProtected(View, 'get_size')
+    security.declareProtected(View, 'getContentType')
+    security.declareProtected(ftp_access, 'manage_FTPstat')
+    security.declareProtected(ftp_access, 'manage_FTPlist')
+    security.declareProtected(ftp_access, 'manage_FTPget')
+    security.declareProtected(delete_objects, 'DELETE')
 
     _properties=({'id':'title', 'type': 'string'},
+                 {'id':'alt', 'type':'string'},
                  {'id':'content_type', 'type':'string','mode':'w'},
                  {'id':'height', 'type':'string'},
                  {'id':'width', 'type':'string'},
@@ -756,14 +768,22 @@ class Image(File):
 
     manage_editForm  =DTMLFile('dtml/imageEdit',globals(),
                                Kind='Image',kind='image')
-    view_image_or_file =DTMLFile('dtml/imageView',globals())
     manage_editForm._setName('manage_editForm')
+
+    security.declareProtected(View, 'view_image_or_file')
+    view_image_or_file =DTMLFile('dtml/imageView',globals())
+
+    security.declareProtected(view_management_screens, 'manage')
+    security.declareProtected(view_management_screens, 'manage_main')
     manage=manage_main=manage_editForm
     manage_uploadForm=manage_editForm
 
-    # private
-    update_data__roles__=()
+    security.declarePrivate('update_data')
     def update_data(self, data, content_type=None, size=None):
+        if isinstance(data, unicode):
+            raise TypeError('Data can only be str or file-like.  '
+                            'Unicode objects are expressly forbidden.')
+        
         if size is None: size=len(data)
 
         self.size=size
@@ -786,6 +806,7 @@ class Image(File):
     def __str__(self):
         return self.tag()
 
+    security.declareProtected(View, 'tag')
     def tag(self, height=None, width=None, alt=None,
             scale=0, xscale=0, yscale=0, css_class=None, title=None, **args):
         """
