@@ -1,4 +1,8 @@
 import unittest
+import Testing
+import Zope2
+Zope2.startup()
+
 import cStringIO
 
 import transaction
@@ -128,6 +132,16 @@ class TestCopySupport( CopySupportTestBase ):
 
         self._cleanApp()
 
+    def test_z3interfaces(self):
+        from OFS.CopySupport import CopyContainer
+        from OFS.CopySupport import CopySource
+        from OFS.interfaces import ICopyContainer
+        from OFS.interfaces import ICopySource
+        from zope.interface.verify import verifyClass
+
+        verifyClass(ICopyContainer, CopyContainer)
+        verifyClass(ICopySource, CopySource)
+
     def testRename( self ):
         self.failUnless( 'file' in self.folder1.objectIds() )
         self.folder1.manage_renameObject( id='file', new_id='filex' )
@@ -333,7 +347,7 @@ class TestCopySupportSecurity( CopySupportTestBase ):
             if ce_regex is not None:
 
                 pattern = re.compile( ce_regex, re.DOTALL )
-                if pattern.search( e ) is None:
+                if pattern.search(str(e)) is None:
                     self.fail( "Paste failed; didn't match pattern:\n%s" % e )
 
             else:
@@ -404,8 +418,6 @@ class TestCopySupportSecurity( CopySupportTestBase ):
 
     def test_copy_cant_create_target_metatype_not_supported( self ):
 
-        from OFS.CopySupport import CopyError
-
         folder1, folder2 = self._initFolders()
         folder2.all_meta_types = ()
 
@@ -435,8 +447,6 @@ class TestCopySupportSecurity( CopySupportTestBase ):
 
     def test_move_cant_read_source( self ):
 
-        from OFS.CopySupport import CopyError
-
         folder1, folder2 = self._initFolders()
         folder2.all_meta_types = FILE_META_TYPES
 
@@ -455,8 +465,6 @@ class TestCopySupportSecurity( CopySupportTestBase ):
 
     def test_move_cant_create_target_metatype_not_supported( self ):
 
-        from OFS.CopySupport import CopyError
-
         folder1, folder2 = self._initFolders()
         folder2.all_meta_types = ()
 
@@ -470,8 +478,6 @@ class TestCopySupportSecurity( CopySupportTestBase ):
 
     def test_move_cant_create_target_metatype_not_allowed( self ):
 
-        from OFS.CopySupport import CopyError
-
         folder1, folder2 = self._initFolders()
         folder2.all_meta_types = FILE_META_TYPES
 
@@ -489,7 +495,6 @@ class TestCopySupportSecurity( CopySupportTestBase ):
 
     def test_move_cant_delete_source( self ):
 
-        from OFS.CopySupport import CopyError
         from AccessControl.Permissions import delete_objects as DeleteObjects
 
         folder1, folder2 = self._initFolders()
