@@ -20,7 +20,7 @@ Python code.
 __version__='$Revision: 1.56 $'[11:-2]
 
 import sys, os, traceback, re, marshal, new
-import logging
+from logging import getLogger
 from Globals import DTMLFile, MessageDialog, package_home
 import AccessControl, OFS, RestrictedPython
 from Acquisition import aq_parent
@@ -34,10 +34,12 @@ from AccessControl import getSecurityManager
 from OFS.History import Historical, html_diff
 from OFS.Cache import Cacheable
 from AccessControl.ZopeGuards import get_safe_globals, guarded_getattr
+from AccessControl.requestmethod import postonly
+
 from zExceptions import Forbidden
 import Globals
 
-LOG = logging.getLogger('Script (Python)')
+LOG = getLogger('PythonScripts')
 
 # Track the Python bytecode version
 import imp
@@ -322,7 +324,11 @@ class PythonScript(Script, Historical, Cacheable):
         g['__file__'] = getattr(self, '_filepath', None) or self.get_filepath()
         f = new.function(fcode, g, None, fadefs)
 
-        result = f(*args, **kw)
+        try:
+            result = f(*args, **kw)
+        except SystemExit:
+            raise ValueError('SystemExit can not be raised within a PythonScript')
+
         if keyset is not None:
             # Store the result in the cache.
             self.ZCacheable_set(result, keywords=keyset)
@@ -359,6 +365,7 @@ class PythonScript(Script, Historical, Cacheable):
       'manage_proxyForm', 'manage_proxy')
 
     manage_proxyForm = DTMLFile('www/pyScriptProxy', globals())
+    @postonly
     def manage_proxy(self, roles=(), REQUEST=None):
         "Change Proxy Roles"
         self._validateProxy(roles)
@@ -416,7 +423,7 @@ class PythonScript(Script, Historical, Cacheable):
                 k = k.strip().lower()
                 v = v.strip()
                 if not mdata.has_key(k):
-                    SyntaxError, 'Unrecognized header line "%s"' % line
+                    raise SyntaxError, 'Unrecognized header line "%s"' % line
                 if v == mdata[k]:
                     # Unchanged value
                     continue
@@ -440,7 +447,7 @@ class PythonScript(Script, Historical, Cacheable):
             else:
                 self._makeFunction()
         except:
-            LOG.error('write failed', exc_info=True)
+            LOG.error('write failed', exc_info=sys.exc_info())
             raise
 
     def manage_FTPget(self):
