@@ -12,25 +12,33 @@
 ##############################################################################
 __doc__='''Generic Database Connection Support
 
-$Id: Connection.py 71155 2006-11-17 15:55:05Z andreasjung $'''
+$Id: Connection.py 96951 2009-02-21 15:28:09Z andreasjung $'''
 __version__='$Revision: 1.39 $'[11:-2]
 
-import logging
-logger = logging.getLogger('Shared.DC.ZRDB.Connection')
-
+from logging import getLogger
 import Globals, OFS.SimpleItem, AccessControl.Role, Acquisition, sys
 from DateTime import DateTime
 from App.Dialogs import MessageDialog
 from Globals import DTMLFile
+from Globals import InitializeClass
+from AccessControl import ClassSecurityInfo
+from AccessControl.Permissions import view_management_screens
+from AccessControl.Permissions import change_database_connections
+from AccessControl.Permissions import test_database_connections
+from AccessControl.Permissions import open_close_database_connection
 from string import find, join, split
 from Aqueduct import custom_default_report
 from cStringIO import StringIO
 from Results import Results
 from sys import exc_info
+
 from cgi import escape
 import DocumentTemplate, RDB
 from zExceptions import BadRequest
 
+
+LOG = getLogger('ZRDB.Connection')
+
 class Connection(
     Globals.Persistent,
     AccessControl.Role.RoleManager,
@@ -38,6 +46,8 @@ class Connection(
     Acquisition.Implicit,
     ):
 
+    security = ClassSecurityInfo()
+
     # Specify definitions for tabs:
     manage_options=(
         (
@@ -49,15 +59,6 @@ class Connection(
         +OFS.SimpleItem.Item.manage_options
         )
 
-    # Specify how individual operations add up to "permissions":
-    __ac_permissions__=(
-        ('View management screens', ('manage_main',)),
-        ('Change Database Connections', ('manage_edit',)),
-        ('Test Database Connections', ('manage_testForm','manage_test')),
-        ('Open/Close Database Connection',
-         ('manage_open_connection', 'manage_close_connection')),
-        )
-
     _v_connected=''
     connection_string=''
 
@@ -70,8 +71,8 @@ class Connection(
         if self.connection_string:
             try: self.connect(self.connection_string)
             except:
-                logger.error('Error connecting to relational database.',
-                             exc_info=True)
+                LOG.error('Error connecting to relational database.',
+                          exc_info=exc_info())
 
     def title_and_id(self):
         s=Connection.inheritedAttribute('title_and_id')(self)
@@ -97,6 +98,8 @@ class Connection(
         if check: self.connect(connection_string)
 
     manage_properties=DTMLFile('dtml/connectionEdit', globals())
+
+    security.declareProtected(change_database_connections, 'manage_edit')
     def manage_edit(self, title, connection_string, check=None, REQUEST=None):
         """Change connection
         """
@@ -108,7 +111,10 @@ class Connection(
                 action ='./manage_main',
                 )
 
+    security.declareProtected(test_database_connections, 'manage_testForm')
     manage_testForm=DTMLFile('dtml/connectionTestForm', globals())
+
+    security.declareProtected(test_database_connections, 'manage_test')
     def manage_test(self, query, REQUEST=None):
         "Executes the SQL in parameter 'query' and returns results"
         dbc=self()      #get our connection
@@ -142,20 +148,25 @@ class Connection(
         return report
 
 
+    security.declareProtected(view_management_screens, 'manage_main')
     manage_main=DTMLFile('dtml/connectionStatus', globals())
 
+    security.declareProtected(open_close_database_connection,
+                              'manage_close_connection')
     def manage_close_connection(self, REQUEST=None):
         " "
         try: 
             if hasattr(self,'_v_database_connection'):
                 self._v_database_connection.close()
         except:
-            logger.error('Error closing relational database connection.',
-                         exc_info=True)
+            LOG.error('Error closing relational database connection.',
+                      exc_info=exc_info())
         self._v_connected=''
         if REQUEST is not None:
             return self.manage_main(self, REQUEST)
 
+    security.declareProtected(open_close_database_connection,
+                              'manage_open_connection')
     def manage_open_connection(self, REQUEST=None):
         " "
         self.connect(self.connection_string)
@@ -180,9 +191,9 @@ class Connection(
             except:
                 t, v, tb = sys.exc_info()
                 raise BadRequest, (
-                    '<strong>Invalid connection string: </strong><CODE>%s</CODE><br>\n'
+                    '<strong>Error connecting to DB.</strong><br>\n'
                     '<!--\n%s\n%s\n-->\n'
-                    % (s,t,v)), tb
+                    % (t,v)), tb
         finally: tb=None
         self._v_connected=DateTime()
 
@@ -191,3 +202,5 @@ class Connection(
     def sql_quote__(self, v):
         if find(v,"\'") >= 0: v=join(split(v,"\'"),"''")
         return "'%s'" % v
+
+InitializeClass(Connection)
