@@ -64,7 +64,7 @@ def log(message, level=logging.INFO, label=None, exc_info=False):
     logger.log(level, message, exc_info=exc_info)
 
 class StorageServerError(StorageError):
-    """Error reported when an unpickleable exception is raised."""
+    """Error reported when an unpicklable exception is raised."""
 
 class ZEOStorage:
     """Proxy to underlying storage for a single remote client."""
@@ -98,7 +98,7 @@ class ZEOStorage:
         for func in self.extensions:
             self._extensions[func.func_name] = None
 
-    def finish_auth(self, authenticated):
+    def _finish_auth(self, authenticated):
         if not self.auth_realm:
             return 1
         self.authenticated = authenticated
@@ -151,6 +151,11 @@ class ZEOStorage:
         self.load = self.storage.load
         self.loadSerial = self.storage.loadSerial
         self.modifiedInVersion = self.storage.modifiedInVersion
+        record_iternext = getattr(self.storage, 'record_iternext', None)
+        if record_iternext is not None:
+            self.record_iternext = record_iternext
+            
+            
         try:
             fn = self.storage.getExtensionMethods
         except AttributeError:
@@ -206,15 +211,15 @@ class ZEOStorage:
         immediately after authentication is finished.
         """
         if self.auth_realm and not self.authenticated:
-            raise AuthError, "Client was never authenticated with server!"
+            raise AuthError("Client was never authenticated with server!")
 
         if self.storage is not None:
             self.log("duplicate register() call")
-            raise ValueError, "duplicate register() call"
+            raise ValueError("duplicate register() call")
         storage = self.server.storages.get(storage_id)
         if storage is None:
             self.log("unknown storage_id: %s" % storage_id)
-            raise ValueError, "unknown storage: %s" % storage_id
+            raise ValueError("unknown storage: %s" % storage_id)
 
         if not read_only and (self.read_only or storage.isReadOnly()):
             raise ReadOnlyError()
@@ -233,6 +238,7 @@ class ZEOStorage:
                 'supportsUndo': self.storage.supportsUndo(),
                 'supportsVersions': self.storage.supportsVersions(),
                 'extensionMethods': self.getExtensionMethods(),
+                'supports_record_iternext': hasattr(self, 'record_iternext'),
                 }
 
     def get_size_info(self):
@@ -350,6 +356,7 @@ class ZEOStorage:
 
     def new_oids(self, n=100):
         """Return a sequence of n new oids, where n defaults to 100"""
+        n = min(n, 100)
         if self.read_only:
             raise ReadOnlyError()
         if n <= 0:
@@ -628,6 +635,7 @@ class ZEOStorage:
         else:
             return 1
 
+
 class StorageServer:
 
     """The server side implementation of ZEO.
