@@ -12,6 +12,7 @@
 #
 ##############################################################################
 import asyncore
+import atexit
 import errno
 import select
 import sys
@@ -24,7 +25,7 @@ import traceback, time
 import ThreadedAsync
 from ZEO.zrpc import smac
 from ZEO.zrpc.error import ZRPCError, DisconnectedError
-from ZEO.zrpc.marshal import Marshaller
+from ZEO.zrpc.marshal import Marshaller, ServerMarshaller
 from ZEO.zrpc.trigger import trigger
 from ZEO.zrpc.log import short_repr, log
 from ZODB.loglevels import BLATHER, TRACE
@@ -39,6 +40,7 @@ client_timeout_count = 0 # for testing
 client_map = {}
 client_trigger = trigger(client_map)
 client_logger = logging.getLogger('ZEO.zrpc.client_loop')
+atexit.register(client_map.clear)
 
 def client_loop():
     map = client_map
@@ -50,8 +52,12 @@ def client_loop():
     
     while map:
         try:
-            r = e = list(client_map)
-            w = [fd for (fd, obj) in map.iteritems() if obj.writable()]
+            
+            # The next two lines intentionally don't use
+            # iterators. Other threads can close dispatchers, causeing
+            # the socket map to shrink.
+            r = e = client_map.keys()
+            w = [fd for (fd, obj) in map.items() if obj.writable()]
 
             try:
                 r, w, e = select.select(r, w, e, client_timeout)
@@ -106,17 +112,25 @@ def client_loop():
                 _exception(obj)
 
         except:
-            client_logger.critical('The ZEO cient loop failed.',
-                            exc_info=sys.exc_info())
-            for fd, obj in map.items():
-                if obj is client_trigger:
-                    continue
+            if map:
                 try:
-                    obj.mgr.client.close()
-                except:
-                    map.pop(fd, None)
-                    client_logger.critical("Couldn't close a dispatcher.",
+                    client_logger.critical('The ZEO cient loop failed.',
                                            exc_info=sys.exc_info())
+                except:
+                    pass
+
+                for fd, obj in map.items():
+                    if obj is client_trigger:
+                        continue
+                    try:
+                        obj.mgr.client.close()
+                    except:
+                        map.pop(fd, None)
+                        try:
+                            client_logger.critical("Couldn't close a dispatcher.",
+                                                   exc_info=sys.exc_info())
+                        except:
+                            pass
 
 client_thread = threading.Thread(target=client_loop)
 client_thread.setDaemon(True)
@@ -429,6 +443,9 @@ class Connection(smac.SizedMessageAsyncConnection, object):
         self.closed = True
         self.__super_close()
         self.close_trigger()
+        self.replies_cond.acquire()
+        self.replies_cond.notifyAll()
+        self.replies_cond.release()
 
     def close_trigger(self):
         # Overridden by ManagedClientConnection.
@@ -724,7 +741,7 @@ class Connection(smac.SizedMessageAsyncConnection, object):
                                  (msgid, short_repr(reply)), level=TRACE)
                     return reply
                 if self.is_async():
-                    self.replies_cond.wait(10.0)
+                    self.replies_cond.wait()
                 else:
                     self.replies_cond.release()
                     try:
@@ -821,6 +838,7 @@ class ManagedServerConnection(Connection):
     def __init__(self, sock, addr, obj, mgr):
         self.mgr = mgr
         self.__super_init(sock, addr, obj, 'S')
+        self.marshal = ServerMarshaller()
         self.obj.notifyConnected(self)
 
     def handshake(self):
