@@ -12,7 +12,7 @@
 ##############################################################################
 """ Basic ZPublisher request management.
 
-$Id: BaseRequest.py 77228 2007-06-29 09:49:45Z shh $
+$Id: BaseRequest.py 85832 2008-04-28 21:02:02Z tseaver $
 """
 from urllib import quote as urllib_quote
 import xmlrpc
@@ -121,7 +121,10 @@ class DefaultPublishTraverse(object):
                     pass
 
                 # Lastly we try with key access:
-                subobject = object[name]
+                try:
+                    subobject = object[name]
+                except TypeError: # unsubscriptable
+                    raise KeyError(name)
                 
 
         # Ensure that the object has a docstring, or that the parent
