@@ -12,7 +12,7 @@
 ##############################################################################
 '''CGI Response Output formatter
 
-$Id: HTTPResponse.py 75581 2007-05-06 19:47:41Z hannosch $'''
+$Id: HTTPResponse.py 99502 2009-04-26 00:12:23Z ldr $'''
 __version__ = '$Revision: 1.81 $'[11:-2]
 
 import types, os, sys, re
@@ -122,6 +122,10 @@ otherTypes = os.environ.get('DONT_GZIP_MAJOR_MIME_TYPES','').lower()
 if otherTypes:
     uncompressableMimeMajorTypes += tuple(otherTypes.split(','))
 
+_CRLF = re.compile(r'\r[\n]?')
+
+def _scrubHeader(name, value):
+    return ''.join(_CRLF.split(str(name))), ''.join(_CRLF.split(str(value)))
 
 class HTTPResponse(BaseResponse):
     """\
@@ -217,7 +221,7 @@ class HTTPResponse(BaseResponse):
             # It has already been determined.
             return
 
-        if (isinstance(status, types.ClassType)
+        if (isinstance(status, (type, types.ClassType))
          and issubclass(status, Exception)):
             status = status.__name__
 
@@ -242,18 +246,18 @@ class HTTPResponse(BaseResponse):
         if lock:
              self._locked_status = 1
 
-    def setHeader(self, name, value, literal=0):
+    def setHeader(self, name, value, literal=0, scrubbed=False):
         '''\
         Sets an HTTP return header "name" with value "value", clearing
         the previous value set for the header, if one exists. If the
         literal flag is true, the case of the header name is preserved,
         otherwise the header name will be lowercased.'''
-        name = str(name)
-        value = str(value)
+        if not scrubbed:
+            name, value = _scrubHeader(name, value)
         key = name.lower()
         if accumulate_header(key):
             self.accumulated_headers = (
-                "%s%s: %s\n" % (self.accumulated_headers, name, value))
+                "%s%s: %s\r\n" % (self.accumulated_headers, name, value))
             return
         name = literal and name or key
         self.headers[name] = value
@@ -274,10 +278,9 @@ class HTTPResponse(BaseResponse):
         '''\
         Set a new HTTP return header with the given value, while retaining
         any previously set headers with the same name.'''
-        name = str(name)
-        value = str(value)
+        name, value = _scrubHeader(name, value)
         self.accumulated_headers = (
-            "%s%s: %s\n" % (self.accumulated_headers, name, value))
+            "%s%s: %s\r\n" % (self.accumulated_headers, name, value))
 
     __setitem__ = setHeader
 
@@ -395,7 +398,9 @@ class HTTPResponse(BaseResponse):
                         # was ignored anyway, so cache should not
                         # vary on it. Otherwise if not forced, cache should
                         # respect Accept-Encoding client header
-                        self.appendHeader('Vary','Accept-Encoding')
+                        vary = self.getHeader('Vary')
+                        if vary is None or 'Accept-Encoding' not in vary: 
+                            self.appendHeader('Vary','Accept-Encoding')
         return self
 
     def enableHTTPCompression(self,REQUEST={},force=0,disable=0,query=0):
@@ -469,21 +474,20 @@ class HTTPResponse(BaseResponse):
 
         # Encode the Unicode data as requested
 
-        if self.headers.has_key('content-type'):
-            match = charset_re.match(self.headers['content-type'])
+        ct = self.headers.get('content-type')
+        if ct:
+            match = charset_re.match(ct)
             if match:
                 encoding = match.group(1)
                 body = body.encode(encoding)
                 body = fix_xml_preamble(body, encoding)
                 return body
             else:
-
-                ct = self.headers['content-type']
                 if ct.startswith('text/') or ct.startswith('application/'):
                     self.headers['content-type'] = '%s; charset=%s' % (ct, default_encoding)
 
         # Use the default character encoding
-        body = body.encode(default_encoding,'replace')
+        body = body.encode(default_encoding, 'replace')
         body = fix_xml_preamble(body, default_encoding)
         return body
 
@@ -585,16 +589,16 @@ class HTTPResponse(BaseResponse):
         Sets an HTTP return header "name" with value "value",
         appending it following a comma if there was a previous value
         set for the header. '''
-        name = str(name).lower()
-        value = str(value)
+        name, value = _scrubHeader(name, value)
+        name = name.lower()
 
         headers = self.headers
         if headers.has_key(name):
             h = headers[name]
-            h = "%s%s\n\t%s" % (h,delimiter,value)
+            h = "%s%s\r\n\t%s" % (h,delimiter,value)
         else:
             h = value
-        self.setHeader(name,h)
+        self.setHeader(name,h, scrubbed=True)
 
     def isHTML(self, s):
         s = s.lstrip()
@@ -893,7 +897,7 @@ class HTTPResponse(BaseResponse):
         if self.cookies:
             headersl = headersl+self._cookie_list()
         headersl[len(headersl):] = [self.accumulated_headers, body]
-        return '\n'.join(headersl)
+        return '\r\n'.join(headersl)
 
     def write(self,data):
         """\
