@@ -1,46 +1,56 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
-# FOR A PARTICULAR PURPOSE
+# FOR A PARTICULAR PURPOSE.
 #
 ##############################################################################
+"""WebDAV support - null resource objects.
 
-"""WebDAV support - null resource objects."""
+$Id: NullResource.py 87397 2008-06-14 17:45:02Z tseaver $
+"""
 
-__version__='$Revision: 1.43 $'[11:-2]
+import sys
 
-import sys, Globals, davcmds
-import Acquisition, OFS.content_types
-from common import aq_base, tokenFinder, IfParser
-from AccessControl import getSecurityManager
-from Resource import Resource
-from Globals import Persistent, DTMLFile
-from WriteLockInterface import WriteLockInterface
+import Acquisition 
 import OFS.SimpleItem
-from zExceptions import Unauthorized, NotFound, Forbidden, BadRequest
+from Globals import InitializeClass
+from AccessControl import getSecurityManager
+from AccessControl import ClassSecurityInfo
+from AccessControl.Permissions import view as View
+from AccessControl.Permissions import add_folders
+from AccessControl.Permissions import webdav_lock_items
+from AccessControl.Permissions import webdav_unlock_items
+from Globals import Persistent, DTMLFile
+from OFS.CopySupport import CopyError
 from zExceptions import MethodNotAllowed
+from zExceptions import Unauthorized, NotFound, Forbidden, BadRequest
+
+import davcmds
+from common import aq_base, tokenFinder, IfParser
 from common import isDavCollection
 from common import Locked, Conflict, PreconditionFailed, UnsupportedMediaType
-from OFS.CopySupport import CopyError
+from interfaces import IWriteLock
+from Resource import Resource
+from WriteLockInterface import WriteLockInterface
+
+from zope.contenttype import guess_content_type
+
 
 class NullResource(Persistent, Acquisition.Implicit, Resource):
+
     """Null resources are used to handle HTTP method calls on
     objects which do not yet exist in the url namespace."""
 
     __implements__ = (WriteLockInterface,)
     __null_resource__=1
 
-    __ac_permissions__=(
-        ('View',                             ('HEAD',)),
-        ('Add Folders',                      ('MKCOL',)),
-        ('WebDAV Lock items',                ('LOCK',)),
-    )
+    security = ClassSecurityInfo()
 
     def __init__(self, parent, name, request=None):
         self.__name__=name
@@ -57,9 +67,11 @@ class NullResource(Persistent, Acquisition.Implicit, Resource):
             raise Conflict, 'Collection ancestors must already exist.'
         raise NotFound, 'The requested resource was not found.'
 
+    security.declareProtected(View, 'HEAD')
     def HEAD(self, REQUEST, RESPONSE):
         """Retrieve resource information without a response message body."""
         self.dav__init(REQUEST, RESPONSE)
+        RESPONSE.setBody('', lock=True)
         raise NotFound, 'The requested resource does not exist.'
 
     # Most methods return 404 (Not Found) for null resources.
@@ -82,7 +94,7 @@ class NullResource(Persistent, Acquisition.Implicit, Resource):
             ob=File(name, '', body, content_type=typ)
         return ob
 
-    PUT__roles__ = ('Anonymous',)
+    security.declarePublic('PUT')
     def PUT(self, REQUEST, RESPONSE):
         """Create a new non-collection resource.
         """
@@ -94,7 +106,9 @@ class NullResource(Persistent, Acquisition.Implicit, Resource):
         parent = self.__parent__
 
         ifhdr = REQUEST.get_header('If', '')
-        if WriteLockInterface.isImplementedBy(parent) and parent.wl_isLocked():
+        if (IWriteLock.providedBy(parent) or
+                WriteLockInterface.isImplementedBy(parent)) and \
+                parent.wl_isLocked():
             if ifhdr:
                 parent.dav__simpleifhandler(REQUEST, RESPONSE, col=1)
             else:
@@ -132,7 +146,7 @@ class NullResource(Persistent, Acquisition.Implicit, Resource):
 
         typ=REQUEST.get_header('content-type', None)
         if typ is None:
-            typ, enc=OFS.content_types.guess_content_type(name, body)
+            typ, enc=guess_content_type(name, body)
 
         factory = getattr(parent, 'PUT_factory', self._default_PUT_factory )
         ob = factory(name, typ, body)
@@ -157,6 +171,7 @@ class NullResource(Persistent, Acquisition.Implicit, Resource):
         RESPONSE.setBody('')
         return RESPONSE
 
+    security.declareProtected(add_folders, 'MKCOL')
     def MKCOL(self, REQUEST, RESPONSE):
         """Create a new collection resource."""
         self.dav__init(REQUEST, RESPONSE)
@@ -172,7 +187,9 @@ class NullResource(Persistent, Acquisition.Implicit, Resource):
             raise Forbidden, 'Cannot create collection at this location.'
 
         ifhdr = REQUEST.get_header('If', '')
-        if WriteLockInterface.isImplementedBy(parent) and parent.wl_isLocked():
+        if (IWriteLock.providedBy(parent) or
+                WriteLockInterface.isImplementedBy(parent)) and \
+                parent.wl_isLocked():
             if ifhdr:
                 parent.dav__simpleifhandler(REQUEST, RESPONSE, col=1)
             else:
@@ -190,6 +207,7 @@ class NullResource(Persistent, Acquisition.Implicit, Resource):
         RESPONSE.setBody('')
         return RESPONSE
 
+    security.declareProtected(webdav_lock_items, 'LOCK')
     def LOCK(self, REQUEST, RESPONSE):
         """ LOCK on a Null Resource makes a LockNullResource instance """
         self.dav__init(REQUEST, RESPONSE)
@@ -202,7 +220,9 @@ class NullResource(Persistent, Acquisition.Implicit, Resource):
         name = self.__name__
         parent = self.__parent__
 
-        if WriteLockInterface.isImplementedBy(parent) and parent.wl_isLocked():
+        if (IWriteLock.providedBy(parent) or
+                WriteLockInterface.isImplementedBy(parent)) and \
+                parent.wl_isLocked():
             if ifhdr:
                 parent.dav__simpleifhandler(REQUEST, RESPONSE, col=1)
             else:
@@ -234,13 +254,12 @@ class NullResource(Persistent, Acquisition.Implicit, Resource):
         else:
             # The command was succesful
             lock = locknull.wl_getLock(token)
-            RESPONSE.setStatus(200)                         
+            RESPONSE.setStatus(200)
             RESPONSE.setHeader('Content-Type', 'text/xml; charset="utf-8"')
             RESPONSE.setHeader('Lock-Token', 'opaquelocktoken:' + token)
             RESPONSE.setBody(lock.asXML())
 
-
-Globals.default__class_init__(NullResource)
+InitializeClass(NullResource)
 
 
 class LockNullResource(NullResource, OFS.SimpleItem.Item_w__name__):
@@ -253,17 +272,14 @@ class LockNullResource(NullResource, OFS.SimpleItem.Item_w__name__):
     __locknull_resource__ = 1
     meta_type = 'WebDAV LockNull Resource'
 
-    __ac_permissions__ = (
-        ('WebDAV Unlock items',              ('UNLOCK',)),
-        ('View',                             ('manage_main',
-                                              'manage_workspace', 'manage')),
-        ('Add Folders',                      ('MKCOL',)),
-        ('WebDAV Lock items',                ('LOCK',)),
-        )
+    security = ClassSecurityInfo()
 
     manage_options = ({'label': 'Info', 'action': 'manage_main'},)
 
+    security.declareProtected(View, 'manage')
+    security.declareProtected(View, 'manage_main')
     manage = manage_main = DTMLFile('dtml/locknullmain', globals())
+    security.declareProtected(View, 'manage_workspace')
     manage_workspace = manage
     manage_main._setName('manage_main')  # explicit
 
@@ -278,7 +294,7 @@ class LockNullResource(NullResource, OFS.SimpleItem.Item_w__name__):
         self.id = self.__name__ = name
         self.title = "LockNull Resource '%s'" % name
 
-    title_or_id__roles__=None
+    security.declarePublic('title_or_id')
     def title_or_id(self):
         return 'Foo'
 
@@ -286,6 +302,7 @@ class LockNullResource(NullResource, OFS.SimpleItem.Item_w__name__):
         """Retrieve properties defined on the resource."""
         return Resource.PROPFIND(self, REQUEST, RESPONSE)
 
+    security.declareProtected(webdav_lock_items, 'LOCK')
     def LOCK(self, REQUEST, RESPONSE):
         """ A Lock command on a LockNull resource should only be a
         refresh request (one without a body) """
@@ -323,6 +340,7 @@ class LockNullResource(NullResource, OFS.SimpleItem.Item_w__name__):
         return RESPONSE
 
 
+    security.declareProtected(webdav_unlock_items, 'UNLOCK')
     def UNLOCK(self, REQUEST, RESPONSE):
         """ Unlocking a Null Resource removes it from its parent """
         self.dav__init(REQUEST, RESPONSE)
@@ -349,7 +367,7 @@ class LockNullResource(NullResource, OFS.SimpleItem.Item_w__name__):
             RESPONSE.setStatus(204)
         return RESPONSE
 
-    PUT__roles__ = ('Anonymous',)
+    security.declarePublic('PUT')
     def PUT(self, REQUEST, RESPONSE):
         """ Create a new non-collection resource, deleting the LockNull
         object from the container before putting the new object in. """
@@ -369,7 +387,9 @@ class LockNullResource(NullResource, OFS.SimpleItem.Item_w__name__):
         # First we need to see if the parent of the locknull is locked, and
         # if the user owns that lock (checked by handling the information in
         # the If header).
-        if WriteLockInterface.isImplementedBy(parent) and parent.wl_isLocked():
+        if (IWriteLock.providedBy(parent) or
+                WriteLockInterface.isImplementedBy(parent)) and \
+                parent.wl_isLocked():
             itrue = parent.dav__simpleifhandler(REQUEST, RESPONSE, 'PUT',
                                                 col=1, url=parenturl,
                                                 refresh=1)
@@ -392,7 +412,7 @@ class LockNullResource(NullResource, OFS.SimpleItem.Item_w__name__):
         body = REQUEST.get('BODY', '')
         typ = REQUEST.get_header('content-type', None)
         if typ is None:
-            typ, enc = OFS.content_types.guess_content_type(name, body)
+            typ, enc = guess_content_type(name, body)
 
         factory = getattr(parent, 'PUT_factory', self._default_PUT_factory)
         ob = (factory(name, typ, body) or
@@ -407,7 +427,8 @@ class LockNullResource(NullResource, OFS.SimpleItem.Item_w__name__):
             raise Forbidden, sys.exc_info()[1]
 
         # Put the locks on the new object
-        if not WriteLockInterface.isImplementedBy(ob):
+        if not (IWriteLock.providedBy(ob) or
+                WriteLockInterface.isImplementedBy(ob)):
             raise MethodNotAllowed, (
                 'The target object type cannot be locked')
         for token, lock in locks:
@@ -421,6 +442,7 @@ class LockNullResource(NullResource, OFS.SimpleItem.Item_w__name__):
         RESPONSE.setBody('')
         return RESPONSE
 
+    security.declareProtected(add_folders, 'MKCOL')
     def MKCOL(self, REQUEST, RESPONSE):
         """ Create a new Collection (folder) resource.  Since this is being
         done on a LockNull resource, this also involves removing the LockNull
@@ -439,7 +461,9 @@ class LockNullResource(NullResource, OFS.SimpleItem.Item_w__name__):
 
         # If the parent object is locked, that information should be in the
         # if-header if the user owns a lock on the parent
-        if WriteLockInterface.isImplementedBy(parent) and parent.wl_isLocked():
+        if (IWriteLock.providedBy(parent) or
+                WriteLockInterface.isImplementedBy(parent)) and \
+                parent.wl_isLocked():
             itrue = parent.dav__simpleifhandler(REQUEST, RESPONSE, 'MKCOL',
                                                 col=1, url=parenturl,
                                                 refresh=1)
@@ -466,5 +490,4 @@ class LockNullResource(NullResource, OFS.SimpleItem.Item_w__name__):
         RESPONSE.setBody('')
         return RESPONSE
 
-
-Globals.default__class_init__(LockNullResource)
+InitializeClass(LockNullResource)
