@@ -13,14 +13,28 @@
 ##############################################################################
 """Browser views
 
-$Id: __init__.py 25177 2004-06-02 13:17:31Z jim $
+$Id: __init__.py 67630 2006-04-27 00:54:03Z jim $
 """
+__docformat__ = 'restructuredtext'
 
-from zope.app import zapi
-from zope.app.publisher.browser import BrowserView
+import zope.component
+from zope.publisher.interfaces.browser import IBrowserRequest
+from zope.publisher.browser import BrowserView
 
 from zope.app.tree.interfaces import ITreeStateEncoder
 from zope.app.tree.node import Node
+from zope.app.rotterdam import Rotterdam
+
+class IStaticTreeLayer(IBrowserRequest):
+    """Layer that we can register our own navigation macro for."""
+
+class IStaticTreeSkin(IStaticTreeLayer, Rotterdam):
+    """Skin based on Rotterdam that includes the static tree
+    navigation macro."""
+
+# BBB 2006/02/18, to be removed after 12 months
+import zope.app.skins
+zope.app.skins.set('StaticTree', IStaticTreeSkin)
 
 class StatefulTreeView(BrowserView):
 
@@ -31,7 +45,7 @@ class StatefulTreeView(BrowserView):
             root = self.context
         expanded_nodes = []
         if tree_state is not None:
-            encoder = zapi.getUtility(ITreeStateEncoder)
+            encoder = zope.component.getUtility(ITreeStateEncoder)
             expanded_nodes = encoder.decodeTreeState(tree_state)
         node = Node(root, expanded_nodes, filter)
         node.expand()
