@@ -13,7 +13,7 @@
 ##############################################################################
 """Default 'ISecurityManagement' and 'IInteractionManagement' implementation
 
-$Id: management.py 40098 2005-11-14 19:14:00Z poster $
+$Id: management.py 84163 2008-02-23 01:15:51Z tseaver $
 """
 # Special system user that has all permissions
 # zope.security.simplepolicies needs it
@@ -78,17 +78,13 @@ def getInteraction():
 
 def newInteraction(*participations):
     """Start a new interaction."""
-    
-    
+
     if queryInteraction() is not None:
-        stack = queryInteraction()._newInteraction_called_from
         raise AssertionError("newInteraction called"
-                             " while another interaction is active:\n%s"
-                             % "".join(traceback.format_list(stack)))
+                             " while another interaction is active.")
 
     interaction = getSecurityPolicy()(*participations)
 
-    interaction._newInteraction_called_from = traceback.extract_stack()
     thread_local.interaction = interaction
 
 def endInteraction():
