@@ -13,22 +13,22 @@
 ##############################################################################
 """Security Checker tests
 
-$Id: test_checker.py 27188 2004-08-20 17:02:03Z jim $
+$Id: test_checker.py 67630 2006-04-27 00:54:03Z jim $
 """
 from unittest import TestCase, TestSuite, main, makeSuite
 from zope.interface import implements
 from zope.interface.verify import verifyObject
-from zope.security.checker import Checker, NamesChecker, CheckerPublic
 from zope.testing.cleanup import CleanUp
-from zope.security.interfaces import ISecurityPolicy
+from zope.proxy import getProxiedObject
+from zope.security.interfaces import ISecurityPolicy, Unauthorized
 from zope.security.interfaces import Forbidden, ForbiddenAttribute
-from zope.security.interfaces import Unauthorized
 from zope.security.management import setSecurityPolicy, newInteraction
 from zope.security.management import endInteraction, getInteraction
-from zope.security.proxy import removeSecurityProxy
-from zope.security.proxy import getChecker
-from zope.security.checker import defineChecker, ProxyFactory
-from zope.security.proxy import Proxy
+from zope.security.proxy import removeSecurityProxy, getChecker, Proxy
+from zope.security.checker import defineChecker, undefineChecker, ProxyFactory
+from zope.security.checker import canWrite, canAccess
+from zope.security.checker import Checker, NamesChecker, CheckerPublic
+from zope.security.checker import BasicTypes, _checkers, NoProxy, _clear
 import types, pickle
 
 class SecurityPolicy(object):
@@ -244,6 +244,24 @@ class Test(TestCase, CleanUp):
             #    proxy2 = checker.proxy(proxy)
             #    self.failUnless(proxy2 is proxy, [proxy, proxy2])
 
+    def testLayeredProxies(self):
+        """Tests that a Proxy will not be re-proxied."""
+        class Base:
+            __Security_checker__ = NamesChecker(['__Security_checker__'])
+        base = Base()
+        checker = Checker({})
+
+        # base is not proxied, so we expect a proxy
+        proxy1 = checker.proxy(base)
+        self.assert_(type(proxy1) is Proxy)
+        self.assert_(getProxiedObject(proxy1) is base)
+
+        # proxy is a proxy, so we don't expect to get another
+        proxy2 = checker.proxy(proxy1)
+        self.assert_(proxy2 is proxy1)
+        self.assert_(getProxiedObject(proxy2) is base)
+
+
     def testMultiChecker(self):
         from zope.interface import Interface
 
@@ -348,6 +366,19 @@ class Test(TestCase, CleanUp):
         self.assert_(type(proxy) is Proxy)
         self.assert_(getChecker(proxy) is specific_checker)
 
+    def test_define_and_undefineChecker(self):
+        class SomeClass(object):
+            pass
+        obj = SomeClass()
+
+        checker = NamesChecker()
+        from zope.security.checker import _defaultChecker, selectChecker
+        self.assert_(selectChecker(obj) is _defaultChecker)
+        defineChecker(SomeClass, checker)
+        self.assert_(selectChecker(obj) is checker)
+        undefineChecker(SomeClass)
+        self.assert_(selectChecker(obj) is _defaultChecker)
+
     def test_ProxyFactory_using_proxy(self):
         class SomeClass(object):
             pass
@@ -366,6 +397,81 @@ class Test(TestCase, CleanUp):
         proxy2 = ProxyFactory(proxy1, checker)
         self.assert_(proxy1 is proxy2)
 
+    def test_canWrite_canAccess(self):
+        # the canWrite and canAccess functions are conveniences.  Often code
+        # wants to check if a certain option is open to a user before
+        # presenting it.  If the code relies on a certain permission, the
+        # Zope 3 goal of keeping knowledge of security assertions out of the
+        # code and only in the zcml assertions is broken.  Instead, ask if the
+        # current user canAccess or canWrite some pertinent aspect of the
+        # object.  canAccess is used for both read access on an attribute
+        # and call access to methods.
+
+        # For example, consider this humble pair of class and object.
+        class SomeClass(object):
+            pass
+        obj = SomeClass()
+
+        # We will establish a checker for the class.  This is the standard
+        # name-based checker, and works by specifying two dicts, one for read
+        # and one for write.  Each item in the dictionary should be an
+        # attribute name and the permission required to read or write it.
+
+        # For these tests, the SecurityPolicy defined at the top of this file
+        # is in place.  It is a stub.  Normally, the security policy would
+        # have knowledge of interactions and participants, and would determine
+        # on the basis of the particpants and the object if a certain permission
+        # were authorized.  This stub simply says that the 'test_allowed'
+        # permission is authorized and nothing else is, for any object you pass
+        # it.
+
+        # Therefore, according to the checker created here, the current
+        # 'interaction' (as stubbed out in the security policy) will be allowed
+        # to access and write foo, and access bar.  The interaction is
+        # unauthorized for accessing baz and writing bar.  Any other access or
+        # write is not merely unauthorized but forbidden--including write access
+        # for baz.
+        checker = Checker(
+            {'foo':'test_allowed', # these are the read settings
+             'bar':'test_allowed',
+             'baz':'you_will_not_have_this_permission'},
+            {'foo':'test_allowed', # these are the write settings
+             'bar':'you_will_not_have_this_permission',
+             'bing':'you_will_not_have_this_permission'})
+        defineChecker(SomeClass, checker)
+
+        # so, our hapless interaction may write and access foo...
+        self.assert_(canWrite(obj, 'foo'))
+        self.assert_(canAccess(obj, 'foo'))
+
+        # ...may access, but not write, bar...
+        self.assert_(not canWrite(obj, 'bar'))
+        self.assert_(canAccess(obj, 'bar'))
+
+        # ...and may access baz.
+        self.assert_(not canAccess(obj, 'baz'))
+
+        # there are no security assertions for writing or reading shazam, so
+        # checking these actually raises Forbidden.  The rationale behind
+        # exposing the Forbidden exception is primarily that it is usually
+        # indicative of programming or configuration errors.
+        self.assertRaises(Forbidden, canAccess, obj, 'shazam')
+        self.assertRaises(Forbidden, canWrite, obj, 'shazam')
+
+        # However, we special-case canWrite when an attribute has a Read
+        # setting but no Write setting.  Consider the 'baz' attribute from the
+        # checker above: it is readonly.  All users are forbidden to write
+        # it.  This is a very reasonable configuration.  Therefore, canWrite
+        # will hide the Forbidden exception if and only if there is a
+        # setting for accessing the attribute.
+        self.assert_(not canWrite(obj, 'baz'))
+
+        # The reverse is not true at the moment: an unusal case like the
+        # write-only 'bing' attribute will return a boolean for canWrite,
+        # but canRead will simply raise a Forbidden exception, without checking
+        # write settings.
+        self.assert_(not canWrite(obj, 'bing'))
+        self.assertRaises(Forbidden, canAccess, obj, 'bing')
 
 class TestCheckerPublic(TestCase):
 
@@ -446,7 +552,8 @@ class TestCombinedChecker(TestMixinDecoratedChecker, TestCase):
         self.interaction.permissions['dc_get_permission'] = False
         cc.check_getattr(self.obj, 'both_get_set')
         self.assert_(
-            self.interaction.checkChecked(['dc_get_permission', 'get_permission'])
+            self.interaction.checkChecked(['dc_get_permission',
+                                           'get_permission'])
             )
 
         # This should raise Unauthorized instead of ForbiddenAttribute, since
@@ -460,11 +567,62 @@ class TestCombinedChecker(TestMixinDecoratedChecker, TestCase):
         dc = CombinedChecker(self.overridingChecker, self.originalChecker)
         verifyObject(IChecker, dc)
 
+
+class TestBasicTypes(TestCase):
+
+    def test(self):
+        class MyType(object): pass
+        class MyType2(object): pass
+
+        # When an item is added to the basic types, it should also be added to
+        # the list of checkers.
+        BasicTypes[MyType] = NoProxy
+        self.assert_(MyType in _checkers)
+
+        # If we clear the checkers, the type should still be there
+        _clear()
+        self.assert_(MyType in BasicTypes)
+        self.assert_(MyType in _checkers)
+
+        # Now delete the type from the dictionary, will also delete it from
+        # the checkers
+        del BasicTypes[MyType]
+        self.assert_(MyType not in BasicTypes)
+        self.assert_(MyType not in _checkers)
+
+        # The quick way of adding new types is using update
+        BasicTypes.update({MyType: NoProxy, MyType2: NoProxy})
+        self.assert_(MyType in BasicTypes)
+        self.assert_(MyType2 in BasicTypes)
+        self.assert_(MyType in _checkers)
+        self.assert_(MyType2 in _checkers)
+
+        # Let's remove the two new types
+        del BasicTypes[MyType]
+        del BasicTypes[MyType2]
+
+        # Of course, BasicTypes is a full dictionary. This dictionary is by
+        # default filled with several entries:
+        keys = BasicTypes.keys()
+        keys.sort()
+        self.assert_(bool in keys)
+        self.assert_(int in keys)
+        self.assert_(float in keys)
+        self.assert_(str in keys)
+        self.assert_(unicode in keys)
+        self.assert_(object in keys)
+        # ...
+
+        # Finally, the ``clear()`` method has been deactivated to avoid
+        # unwanted deletions.
+        self.assertRaises(NotImplementedError, BasicTypes.clear)
+
 def test_suite():
     return TestSuite((
         makeSuite(Test),
         makeSuite(TestCheckerPublic),
         makeSuite(TestCombinedChecker),
+        makeSuite(TestBasicTypes),
         ))
 
 if __name__=='__main__':
